//
//  SecretHeroFlutter.swift
//  secrethero_flutter
//
//  Created by Kadir Sancak on 2.03.2024.
//

import Foundation
import Flutter
import SecretHero

internal enum SecretHeroMethods: String {
    case initialize
    case start
    case release
}

internal enum SecretHeroEvents: String {
    case onInitialized
    case onInitializedFailed
    case onOpen
    case onClose
    case onEvent
}

internal class SecretHeroFlutter {
    private let channel: FlutterMethodChannel
    
    init(channel: FlutterMethodChannel) {
        self.channel = channel
    }
    
    func onMethod(method: SecretHeroMethods, params: [String: Any?], result: @escaping FlutterResult) {
        switch method {
            case .initialize: initialize(params: params)
            case .start: start()
            case .release: SecretHero.release()
        }
    }
    
    private func start() {
        guard let vc = UIApplication.shared.keyWindow?.rootViewController?.getPresentedViewController() else {
            return
        }
        SecretHero.start(rootViewController: vc)
    }
    
    private func initialize(params: [String: Any?]) {
        guard let token = params["token"] as? String,
            let userId = params["userId"] as? String,
            let displayName = params["displayName"] as? String else { return }
        let theme = getTheme(params: params["theme"] as? [String : Any?])
        let config = SecretHeroConfig.Builder()
            .setUserId(userId)
            .setDisplayName(displayName)
            .setTheme(theme)
            .build(token: token)
        SecretHero.initialize(config: config, delegate: self)
    }
    
    private func getTheme(params: [String: Any?]?) -> SecretHeroTheme? {
        guard let params = params else { return nil }
        return SecretHeroTheme.Builder()
            .setPrimaryColor(UIColor(hexString: params["primary_color"] as? String))
            .setSecondaryColor(UIColor(hexString: params["secondary_color"] as? String))
            .setHighlightColor(UIColor(hexString: params["highlight_color"] as? String))
            .setButtonColor(UIColor(hexString: params["button_color"] as? String))
            .build()
    }
    
    private func sendEvent(event: SecretHeroEvents, params: [String: Any?] = [:]) {
        channel.invokeMethod(event.rawValue, arguments: params)
    }
}

extension SecretHeroFlutter: SecretHeroDelegate {
    public func onInitialized() {
        DispatchQueue.main.async {
            self.sendEvent(event: .onInitialized)
        }
    }
    
    public func onInitializeFailed(errorMessage: String) {
        DispatchQueue.main.async {
            self.sendEvent(event: .onInitializedFailed, params: ["errorMessage": errorMessage])
        }
    }
    
    public func onOpen() {
        DispatchQueue.main.async {
            self.sendEvent(event: .onOpen)
        }
    }
    
    public func onClose() {
        DispatchQueue.main.async {
            self.sendEvent(event: .onClose)
        }
    }
    
    func onEvent(event: SecretHeroEvent) {
        DispatchQueue.main.async {
            let eventName: String
            switch event {
                case .GameSuccess: eventName = "game_success"
                case .GameOver: eventName = "game_over"
                case .GameStart: eventName = "game_start"
                @unknown default: return
            }
            self.sendEvent(event: .onEvent, params: ["event": eventName])
        }
    }
    
}

extension UIViewController {
    internal func getPresentedViewController() -> UIViewController? {
        guard let vc = self.presentedViewController else {
            return self
        }
        return vc.getPresentedViewController()
    }
}
